@@ -24,6 +24,10 @@ use groupware::{
 use http_proto::HttpResponse;
 use hyper::StatusCode;
 use store::write::{BatchBuilder, now};
+use store::{
+    ValueKey,
+    write::{AlignedBytes, Archive},
+};
 use trc::AddContext;
 use types::{
     acl::Acl,
@@ -455,7 +459,12 @@ async fn copy_event(
 ) -> crate::Result<HttpResponse> {
     // Fetch event
     let event_ = server
-        .get_archive(from_account_id, Collection::CalendarEvent, from_document_id)
+        .store()
+        .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
+            from_account_id,
+            Collection::CalendarEvent,
+            from_document_id,
+        ))
         .await
         .caused_by(trc::location!())?
         .ok_or(DavError::Code(StatusCode::NOT_FOUND))?;
@@ -523,7 +532,12 @@ async fn copy_event(
     let response = if let Some(to_document_id) = to_document_id {
         // Overwrite event on destination
         let event_ = server
-            .get_archive(to_account_id, Collection::CalendarEvent, to_document_id)
+            .store()
+            .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
+                to_account_id,
+                Collection::CalendarEvent,
+                to_document_id,
+            ))
             .await
             .caused_by(trc::location!())?;
         if let Some(event_) = event_ {
@@ -553,6 +567,7 @@ async fn copy_event(
         .commit_batch(batch)
         .await
         .caused_by(trc::location!())?;
+    server.notify_task_queue();
 
     response
 }
@@ -573,7 +588,12 @@ async fn move_event(
 ) -> crate::Result<HttpResponse> {
     // Fetch event
     let event_ = server
-        .get_archive(from_account_id, Collection::CalendarEvent, from_document_id)
+        .store()
+        .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
+            from_account_id,
+            Collection::CalendarEvent,
+            from_document_id,
+        ))
         .await
         .caused_by(trc::location!())?
         .ok_or(DavError::Code(StatusCode::NOT_FOUND))?;
@@ -682,7 +702,12 @@ async fn move_event(
     let response = if let Some(to_document_id) = to_document_id {
         // Overwrite event on destination
         let event_ = server
-            .get_archive(to_account_id, Collection::CalendarEvent, to_document_id)
+            .store()
+            .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
+                to_account_id,
+                Collection::CalendarEvent,
+                to_document_id,
+            ))
             .await
             .caused_by(trc::location!())?;
         if let Some(event_) = event_ {
@@ -712,6 +737,7 @@ async fn move_event(
         .commit_batch(batch)
         .await
         .caused_by(trc::location!())?;
+    server.notify_task_queue();
 
     response
 }
@@ -728,7 +754,12 @@ async fn rename_event(
 ) -> crate::Result<HttpResponse> {
     // Fetch event
     let event_ = server
-        .get_archive(account_id, Collection::CalendarEvent, document_id)
+        .store()
+        .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
+            account_id,
+            Collection::CalendarEvent,
+            document_id,
+        ))
         .await
         .caused_by(trc::location!())?
         .ok_or(DavError::Code(StatusCode::NOT_FOUND))?;
@@ -756,6 +787,7 @@ async fn rename_event(
         .commit_batch(batch)
         .await
         .caused_by(trc::location!())?;
+    server.notify_task_queue();
 
     Ok(HttpResponse::new(StatusCode::CREATED))
 }
@@ -776,7 +808,12 @@ async fn copy_container(
 ) -> crate::Result<HttpResponse> {
     // Fetch calendar
     let calendar_ = server
-        .get_archive(from_account_id, Collection::Calendar, from_document_id)
+        .store()
+        .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
+            from_account_id,
+            Collection::Calendar,
+            from_document_id,
+        ))
         .await
         .caused_by(trc::location!())?
         .ok_or(DavError::Code(StatusCode::NOT_FOUND))?;
@@ -820,7 +857,12 @@ async fn copy_container(
     let to_document_id = if let Some(to_document_id) = to_document_id {
         // Overwrite destination
         let calendar_ = server
-            .get_archive(to_account_id, Collection::Calendar, to_document_id)
+            .store()
+            .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
+                to_account_id,
+                Collection::Calendar,
+                to_document_id,
+            ))
             .await
             .caused_by(trc::location!())?;
         if let Some(calendar_) = calendar_ {
@@ -859,11 +901,12 @@ async fn copy_container(
     let mut required_space = 0;
     for from_child_document_id in from_children_ids {
         if let Some(event_) = server
-            .get_archive(
+            .store()
+            .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
                 from_account_id,
                 Collection::CalendarEvent,
                 from_child_document_id,
-            )
+            ))
             .await?
         {
             let event = event_
@@ -960,6 +1003,7 @@ async fn copy_container(
         .commit_batch(batch)
         .await
         .caused_by(trc::location!())?;
+    server.notify_task_queue();
 
     if !is_overwrite {
         Ok(HttpResponse::new(StatusCode::CREATED))
@@ -979,7 +1023,12 @@ async fn rename_container(
 ) -> crate::Result<HttpResponse> {
     // Fetch calendar
     let calendar_ = server
-        .get_archive(account_id, Collection::Calendar, document_id)
+        .store()
+        .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
+            account_id,
+            Collection::Calendar,
+            document_id,
+        ))
         .await
         .caused_by(trc::location!())?
         .ok_or(DavError::Code(StatusCode::NOT_FOUND))?;
@@ -1000,6 +1049,7 @@ async fn rename_container(
         .commit_batch(batch)
         .await
         .caused_by(trc::location!())?;
+    server.notify_task_queue();
 
     Ok(HttpResponse::new(StatusCode::CREATED))
 }
