@@ -9,20 +9,30 @@ use super::{
     ArchivedTimezone, Calendar, CalendarEvent, CalendarPreferences, DefaultAlert, Timezone,
 };
 use crate::calendar::{
-    ArchivedCalendarEventNotification, ArchivedEventPreferences, CalendarEventNotification,
-    EventPreferences,
+    ArchivedCalendarEventNotification, ArchivedChangedBy, ArchivedEventPreferences,
+    CalendarEventNotification, ChangedBy, EventPreferences,
 };
+use ahash::AHashSet;
 use calcard::icalendar::{
     ArchivedICalendarParameterValue, ArchivedICalendarProperty, ArchivedICalendarValue,
     ICalendarParameterValue, ICalendarProperty, ICalendarValue,
 };
-use common::storage::index::{
-    IndexItem, IndexValue, IndexableAndSerializableObject, IndexableObject,
+use common::storage::index::{IndexValue, IndexableAndSerializableObject, IndexableObject};
+use nlp::language::{
+    Language,
+    detect::{LanguageDetector, MIN_LANGUAGE_SCORE},
+};
+use store::{
+    U32_LEN,
+    search::{CalendarSearchField, IndexDocument, SearchField},
+    write::{IndexPropertyClass, SearchIndex, ValueClass},
+    xxhash_rust::xxh3,
+};
+use types::{
+    acl::AclGrant,
+    collection::SyncCollection,
+    field::{CalendarEventField, CalendarNotificationField},
 };
-use nlp::tokenizers::word::WordTokenizer;
-use std::collections::HashSet;
-use store::backend::MAX_TOKEN_LENGTH;
-use types::{acl::AclGrant, collection::SyncCollection, field::CalendarField};
 
 impl IndexableObject for Calendar {
     fn index_values(&self) -> impl Iterator<Item = IndexValue<'_>> {
@@ -31,9 +41,7 @@ impl IndexableObject for Calendar {
                 value: (&self.acls).into(),
             },
             IndexValue::Quota {
-                used: self.dead_properties.size() as u32
-                    + self.preferences.iter().map(|p| p.size()).sum::<usize>() as u32
-                    + self.name.len() as u32,
+                used: self.size() as u32,
             },
             IndexValue::LogContainer {
                 sync_collection: SyncCollection::Calendar,
@@ -55,9 +63,7 @@ impl IndexableObject for &ArchivedCalendar {
                     .into(),
             },
             IndexValue::Quota {
-                used: self.dead_properties.size() as u32
-                    + self.preferences.iter().map(|p| p.size()).sum::<usize>() as u32
-                    + self.name.len() as u32,
+                used: self.size() as u32,
             },
             IndexValue::LogContainer {
                 sync_collection: SyncCollection::Calendar,
@@ -76,37 +82,19 @@ impl IndexableAndSerializableObject for Calendar {
 impl IndexableObject for CalendarEvent {
     fn index_values(&self) -> impl Iterator<Item = IndexValue<'_>> {
         [
+            IndexValue::SearchIndex {
+                index: SearchIndex::Calendar,
+                hash: self
+                    .hashes()
+                    .chain([self.data.event_range_start() as u64])
+                    .fold(0, |acc, hash| acc ^ hash),
+            },
             IndexValue::Index {
-                field: CalendarField::Uid.into(),
+                field: CalendarEventField::Uid.into(),
                 value: self.data.event.uids().next().into(),
             },
-            IndexValue::Index {
-                field: CalendarField::Start.into(),
-                value: self.data.event_range_start().into(),
-            },
-            IndexValue::Index {
-                field: CalendarField::Created.into(),
-                value: self.created.into(),
-            },
-            IndexValue::Index {
-                field: CalendarField::Updated.into(),
-                value: self.modified.into(),
-            },
-            IndexValue::IndexList {
-                field: CalendarField::Text.into(),
-                value: self
-                    .text()
-                    .map(Into::into)
-                    .collect::<HashSet<IndexItem>>()
-                    .into_iter()
-                    .collect(),
-            },
             IndexValue::Quota {
-                used: self.dead_properties.size() as u32
-                    + self.display_name.as_ref().map_or(0, |n| n.len() as u32)
-                    + self.names.iter().map(|n| n.name.len() as u32).sum::<u32>()
-                    + self.preferences.iter().map(|p| p.size()).sum::<usize>() as u32
-                    + self.size,
+                used: self.size() as u32,
             },
             IndexValue::LogItem {
                 sync_collection: SyncCollection::Calendar,
@@ -120,37 +108,19 @@ impl IndexableObject for CalendarEvent {
 impl IndexableObject for &ArchivedCalendarEvent {
     fn index_values(&self) -> impl Iterator<Item = IndexValue<'_>> {
         [
+            IndexValue::SearchIndex {
+                index: SearchIndex::Calendar,
+                hash: self
+                    .hashes()
+                    .chain([self.data.event_range_start() as u64])
+                    .fold(0, |acc, hash| acc ^ hash),
+            },
             IndexValue::Index {
-                field: CalendarField::Uid.into(),
+                field: CalendarEventField::Uid.into(),
                 value: self.data.event.uids().next().into(),
             },
-            IndexValue::Index {
-                field: CalendarField::Start.into(),
-                value: self.data.event_range_start().into(),
-            },
-            IndexValue::Index {
-                field: CalendarField::Created.into(),
-                value: self.created.to_native().into(),
-            },
-            IndexValue::Index {
-                field: CalendarField::Updated.into(),
-                value: self.modified.to_native().into(),
-            },
-            IndexValue::IndexList {
-                field: CalendarField::Text.into(),
-                value: self
-                    .text()
-                    .map(Into::into)
-                    .collect::<HashSet<IndexItem>>()
-                    .into_iter()
-                    .collect(),
-            },
             IndexValue::Quota {
-                used: self.dead_properties.size() as u32
-                    + self.display_name.as_ref().map_or(0, |n| n.len() as u32)
-                    + self.names.iter().map(|n| n.name.len() as u32).sum::<u32>()
-                    + self.preferences.iter().map(|p| p.size()).sum::<usize>() as u32
-                    + self.size,
+                used: self.size() as u32,
             },
             IndexValue::LogItem {
                 sync_collection: SyncCollection::Calendar,
@@ -170,13 +140,14 @@ impl IndexableAndSerializableObject for CalendarEvent {
 impl IndexableObject for CalendarEventNotification {
     fn index_values(&self) -> impl Iterator<Item = IndexValue<'_>> {
         [
-            IndexValue::Quota { used: self.size },
-            IndexValue::Index {
-                field: CalendarField::Created.into(),
-                value: self.created.into(),
+            IndexValue::Quota {
+                used: self.size() as u32,
             },
-            IndexValue::Index {
-                field: CalendarField::EventId.into(),
+            IndexValue::Property {
+                field: ValueClass::IndexProperty(IndexPropertyClass::Integer {
+                    property: CalendarNotificationField::CreatedToId.into(),
+                    value: self.created as u64,
+                }),
                 value: self.event_id.unwrap_or(u32::MAX).into(),
             },
             IndexValue::LogItem {
@@ -192,14 +163,13 @@ impl IndexableObject for &ArchivedCalendarEventNotification {
     fn index_values(&self) -> impl Iterator<Item = IndexValue<'_>> {
         [
             IndexValue::Quota {
-                used: self.size.to_native(),
+                used: self.size() as u32,
             },
-            IndexValue::Index {
-                field: CalendarField::Created.into(),
-                value: self.created.to_native().into(),
-            },
-            IndexValue::Index {
-                field: CalendarField::EventId.into(),
+            IndexValue::Property {
+                field: ValueClass::IndexProperty(IndexPropertyClass::Integer {
+                    property: CalendarNotificationField::CreatedToId.into(),
+                    value: self.created.to_native() as u64,
+                }),
                 value: self
                     .event_id
                     .as_ref()
@@ -222,6 +192,66 @@ impl IndexableAndSerializableObject for CalendarEventNotification {
     }
 }
 
+impl Calendar {
+    pub fn size(&self) -> usize {
+        self.dead_properties.size()
+            + self.preferences.iter().map(|p| p.size()).sum::<usize>()
+            + self.name.len()
+            + std::mem::size_of::<Calendar>()
+    }
+}
+
+impl ArchivedCalendar {
+    pub fn size(&self) -> usize {
+        self.dead_properties.size()
+            + self.preferences.iter().map(|p| p.size()).sum::<usize>()
+            + self.name.len()
+            + std::mem::size_of::<Calendar>()
+    }
+}
+
+impl CalendarEvent {
+    pub fn size(&self) -> usize {
+        self.dead_properties.size()
+            + self.display_name.as_ref().map_or(0, |n| n.len())
+            + self.names.iter().map(|n| n.name.len()).sum::<usize>()
+            + self.preferences.iter().map(|p| p.size()).sum::<usize>()
+            + self.size as usize
+            + std::mem::size_of::<CalendarEvent>()
+    }
+}
+
+impl ArchivedCalendarEvent {
+    pub fn size(&self) -> usize {
+        self.dead_properties.size()
+            + self.display_name.as_ref().map_or(0, |n| n.len())
+            + self.names.iter().map(|n| n.name.len()).sum::<usize>()
+            + self.preferences.iter().map(|p| p.size()).sum::<usize>()
+            + self.size.to_native() as usize
+            + std::mem::size_of::<CalendarEvent>()
+    }
+}
+
+impl CalendarEventNotification {
+    pub fn size(&self) -> usize {
+        (match &self.changed_by {
+            ChangedBy::PrincipalId(_) => U32_LEN,
+            ChangedBy::CalendarAddress(v) => v.len(),
+        }) + std::mem::size_of::<CalendarEventNotification>()
+            + self.size as usize
+    }
+}
+
+impl ArchivedCalendarEventNotification {
+    pub fn size(&self) -> usize {
+        (match &self.changed_by {
+            ArchivedChangedBy::PrincipalId(_) => U32_LEN,
+            ArchivedChangedBy::CalendarAddress(v) => v.len(),
+        }) + std::mem::size_of::<CalendarEventNotification>()
+            + self.size.to_native() as usize
+    }
+}
+
 impl CalendarPreferences {
     pub fn size(&self) -> usize {
         self.name.len()
@@ -293,7 +323,7 @@ impl ArchivedDefaultAlert {
 }
 
 impl CalendarEvent {
-    pub fn text(&self) -> impl Iterator<Item = String> {
+    pub fn hashes(&self) -> impl Iterator<Item = u64> {
         self.data
             .event
             .components
@@ -310,6 +340,7 @@ impl CalendarEvent {
                             | ICalendarProperty::Comment
                             | ICalendarProperty::Attendee
                             | ICalendarProperty::Organizer
+                            | ICalendarProperty::Uid
                     )
                 })
             })
@@ -327,15 +358,12 @@ impl CalendarEvent {
                         _ => None,
                     }))
             })
-            .flat_map(|v| {
-                WordTokenizer::new(v.strip_prefix("mailto:").unwrap_or(v), MAX_TOKEN_LENGTH)
-            })
-            .map(|t| t.word.into_owned())
+            .map(|v| xxh3::xxh3_64(v.as_bytes()))
     }
 }
 
 impl ArchivedCalendarEvent {
-    pub fn text(&self) -> impl Iterator<Item = String> {
+    pub fn hashes(&self) -> impl Iterator<Item = u64> {
         self.data
             .event
             .components
@@ -352,6 +380,7 @@ impl ArchivedCalendarEvent {
                             | ArchivedICalendarProperty::Comment
                             | ArchivedICalendarProperty::Attendee
                             | ArchivedICalendarProperty::Organizer
+                            | ArchivedICalendarProperty::Uid
                     )
                 })
             })
@@ -369,9 +398,95 @@ impl ArchivedCalendarEvent {
                         _ => None,
                     }))
             })
-            .flat_map(|v| {
-                WordTokenizer::new(v.strip_prefix("mailto:").unwrap_or(v), MAX_TOKEN_LENGTH)
-            })
-            .map(|t| t.word.into_owned())
+            .map(|v| xxh3::xxh3_64(v.as_bytes()))
+    }
+}
+
+impl ArchivedCalendarEvent {
+    pub fn index_document(
+        &self,
+        account_id: u32,
+        document_id: u32,
+        index_fields: &AHashSet<SearchField>,
+        default_language: Language,
+    ) -> IndexDocument {
+        let mut document = IndexDocument::new(SearchIndex::Calendar)
+            .with_account_id(account_id)
+            .with_document_id(document_id);
+
+        if index_fields.is_empty()
+            || index_fields.contains(&SearchField::Calendar(CalendarSearchField::Start))
+        {
+            document.index_integer(CalendarSearchField::Start, self.data.event_range_start());
+        }
+
+        let mut detector = LanguageDetector::new();
+        for component in self
+            .data
+            .event
+            .components
+            .iter()
+            .filter(|e| e.component_type.is_scheduling_object())
+        {
+            for entry in component.entries.iter() {
+                let (is_lang, is_keyword, field) = match entry.name {
+                    ArchivedICalendarProperty::Summary => (true, false, CalendarSearchField::Title),
+                    ArchivedICalendarProperty::Description => {
+                        (true, false, CalendarSearchField::Description)
+                    }
+                    ArchivedICalendarProperty::Location => {
+                        (false, false, CalendarSearchField::Location)
+                    }
+                    ArchivedICalendarProperty::Organizer => {
+                        (false, false, CalendarSearchField::Owner)
+                    }
+                    ArchivedICalendarProperty::Attendee => {
+                        (false, false, CalendarSearchField::Attendee)
+                    }
+                    ArchivedICalendarProperty::Uid => (false, true, CalendarSearchField::Uid),
+                    _ => continue,
+                };
+                let field = SearchField::Calendar(field);
+
+                if index_fields.is_empty() || index_fields.contains(&field) {
+                    for value in entry
+                        .values
+                        .iter()
+                        .filter_map(|v| match v {
+                            ArchivedICalendarValue::Text(v) => Some(v.as_str()),
+                            ArchivedICalendarValue::Uri(uri) => uri.as_str(),
+                            _ => None,
+                        })
+                        .chain(entry.params.iter().filter_map(|p| match &p.value {
+                            ArchivedICalendarParameterValue::Text(v) => Some(v.as_str()),
+                            ArchivedICalendarParameterValue::Uri(uri) => uri.as_str(),
+                            _ => None,
+                        }))
+                    {
+                        let value = value.strip_prefix("mailto:").unwrap_or(value).trim();
+                        let lang = if is_lang {
+                            detector.detect(value, MIN_LANGUAGE_SCORE);
+                            Language::Unknown
+                        } else {
+                            Language::None
+                        };
+
+                        if !is_keyword {
+                            document.index_text(field.clone(), value, lang);
+                        } else {
+                            document.index_keyword(field.clone(), value);
+                        }
+                    }
+                }
+            }
+        }
+
+        document.set_unknown_language(
+            detector
+                .most_frequent_language()
+                .unwrap_or(default_language),
+        );
+
+        document
     }
 }
