@@ -4,23 +4,20 @@
  * SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-SEL
  */
 
-use crate::{JmapMethods, blob::download::BlobDownload, changes::state::StateManager};
+use crate::{blob::download::BlobDownload, changes::state::StateManager};
 use common::{
     Server,
     auth::{AccessToken, ResourceToken},
     storage::index::ObjectIndexBuilder,
 };
 use email::sieve::{
-    ArchivedSieveScript, SieveScript, activate::SieveScriptActivate, delete::SieveScriptDelete,
+    ArchivedSieveScript, SieveScript, delete::SieveScriptDelete, ingest::SieveScriptIngest,
 };
 use http_proto::HttpSessionData;
 use jmap_proto::{
     error::set::{SetError, SetErrorType},
     method::set::{SetRequest, SetResponse},
-    object::{
-        AnyId,
-        sieve::{Sieve, SieveProperty, SieveValue},
-    },
+    object::sieve::{Sieve, SieveProperty, SieveValue},
     references::resolve::ResolveCreatedReference,
     request::{IntoValid, reference::MaybeIdReference},
     types::state::State,
@@ -30,16 +27,13 @@ use rand::distr::Alphanumeric;
 use sieve::compiler::ErrorType;
 use std::future::Future;
 use store::{
-    Serialize,
-    query::Filter,
-    rand::{Rng, rng},
-    write::{Archive, Archiver, BatchBuilder},
+    Serialize, SerializeInfallible, ValueKey, rand::{Rng, rng}, write::{AlignedBytes, Archive, Archiver, BatchBuilder}
 };
 use trc::AddContext;
 use types::{
     blob::{BlobClass, BlobId, BlobSection},
     collection::{Collection, SyncCollection},
-    field::SieveField,
+    field::{PrincipalField, SieveField},
     id::Id,
 };
 
@@ -86,30 +80,34 @@ impl SieveScriptSet for Server {
     ) -> trc::Result<SetResponse<Sieve>> {
         let account_id = request.account_id.document_id();
         let sieve_ids = self
-            .get_document_ids(account_id, Collection::SieveScript)
-            .await?
-            .unwrap_or_default();
+            .document_ids(account_id, Collection::SieveScript, SieveField::Name)
+            .await?;
         let mut ctx = SetContext {
             resource_token: self.get_resource_token(access_token, account_id).await?,
             access_token,
-            response: self
-                .prepare_set_response(
-                    &request,
+            response: SetResponse::from_request(&request, self.core.jmap.set_max_objects)?
+                .with_state(
                     self.assert_state(
                         account_id,
                         SyncCollection::SieveScript,
                         &request.if_in_state,
                     )
                     .await?,
-                )
-                .await?,
+                ),
         };
         let will_destroy = request.unwrap_destroy().into_valid().collect::<Vec<_>>();
 
+        // Validate active script id
+        if let Some(MaybeIdReference::Id(id)) = &request.arguments.on_success_activate_script
+            && !sieve_ids.contains(id.document_id())
+        {
+            request.arguments.on_success_activate_script = None;
+        }
+
         // Process creates
         let mut batch = BatchBuilder::new();
         for (id, object) in request.unwrap_create() {
-            if sieve_ids.len() as usize <= self.core.jmap.sieve_max_scripts {
+            if sieve_ids.len() < access_token.object_quota(Collection::SieveScript) as u64 {
                 match self
                     .sieve_set_item(object, None, &ctx, session.session_id)
                     .await?
@@ -117,7 +115,9 @@ impl SieveScriptSet for Server {
                     Ok((mut builder, Some(blob))) => {
                         // Store blob
                         let sieve = &mut builder.changes_mut().unwrap();
-                        sieve.blob_hash = self.put_blob(account_id, &blob, false).await?.hash;
+                        let (blob_hash, blob_hold) =
+                            self.put_temporary_blob(account_id, &blob, 60).await?;
+                        sieve.blob_hash = blob_hash;
                         let blob_size = sieve.size as usize;
                         let blob_hash = sieve.blob_hash.clone();
 
@@ -130,37 +130,43 @@ impl SieveScriptSet for Server {
                         batch
                             .with_account_id(account_id)
                             .with_collection(Collection::SieveScript)
-                            .create_document(document_id)
-                            .custom(builder.with_tenant_id(&ctx.resource_token))
+                            .with_document(document_id)
+                            .custom(builder.with_access_token(ctx.access_token))
                             .caused_by(trc::location!())?
+                            .clear(blob_hold)
                             .commit_point();
 
+                        let mut result = Map::with_capacity(1)
+                            .with_key_value(SieveProperty::Id, SieveValue::Id(document_id.into()))
+                            .with_key_value(
+                                SieveProperty::BlobId,
+                                SieveValue::BlobId(BlobId {
+                                    hash: blob_hash,
+                                    class: BlobClass::Linked {
+                                        account_id,
+                                        collection: Collection::SieveScript.into(),
+                                        document_id,
+                                    },
+                                    section: BlobSection {
+                                        size: blob_size,
+                                        ..Default::default()
+                                    }
+                                    .into(),
+                                }),
+                            );
+
+                        // Update active script if needed
+                        if let Some(MaybeIdReference::Reference(id_ref)) =
+                            &request.arguments.on_success_activate_script
+                            && id_ref == &id
+                        {
+                            request.arguments.on_success_activate_script =
+                                Some(MaybeIdReference::Id(Id::from(document_id)));
+                            result.insert_unchecked(SieveProperty::IsActive, true);
+                        }
+
                         // Add result with updated blobId
-                        ctx.response.created.insert(
-                            id,
-                            Map::with_capacity(1)
-                                .with_key_value(
-                                    SieveProperty::Id,
-                                    SieveValue::Id(document_id.into()),
-                                )
-                                .with_key_value(
-                                    SieveProperty::BlobId,
-                                    SieveValue::BlobId(BlobId {
-                                        hash: blob_hash,
-                                        class: BlobClass::Linked {
-                                            account_id,
-                                            collection: Collection::SieveScript.into(),
-                                            document_id,
-                                        },
-                                        section: BlobSection {
-                                            size: blob_size,
-                                            ..Default::default()
-                                        }
-                                        .into(),
-                                    }),
-                                )
-                                .into(),
-                        );
+                        ctx.response.created.insert(id, result.into());
                     }
                     Err(err) => {
                         ctx.response.not_created.append(id, err);
@@ -191,7 +197,12 @@ impl SieveScriptSet for Server {
             // Obtain sieve script
             let document_id = id.document_id();
             if let Some(sieve_) = self
-                .get_archive(account_id, Collection::SieveScript, document_id)
+                .store()
+                .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
+                    account_id,
+                    Collection::SieveScript,
+                    document_id,
+                ))
                 .await?
             {
                 let sieve = sieve_
@@ -212,12 +223,15 @@ impl SieveScriptSet for Server {
                         batch
                             .with_account_id(account_id)
                             .with_collection(Collection::SieveScript)
-                            .update_document(document_id);
+                            .with_document(document_id);
 
                         let blob_id = if let Some(blob) = blob {
                             // Store blob
                             let sieve = &mut builder.changes_mut().unwrap();
-                            sieve.blob_hash = self.put_blob(account_id, &blob, false).await?.hash;
+                            let (blob_hash, blob_hold) =
+                                self.put_temporary_blob(account_id, &blob, 60).await?;
+                            sieve.blob_hash = blob_hash;
+                            batch.clear(blob_hold);
 
                             BlobId {
                                 hash: sieve.blob_hash.clone(),
@@ -239,19 +253,35 @@ impl SieveScriptSet for Server {
 
                         // Write record
                         batch
-                            .custom(builder.with_tenant_id(&ctx.resource_token))
+                            .custom(builder.with_access_token(ctx.access_token))
                             .caused_by(trc::location!())?
                             .commit_point();
 
-                        // Add result with updated blobId
+                        // Update blobId property if needed
+                        let mut result = Map::with_capacity(1);
+                        if let Some(blob_id) = blob_id {
+                            result.insert_unchecked(
+                                SieveProperty::BlobId,
+                                SieveValue::BlobId(blob_id),
+                            );
+                        }
+
+                        // Add active script property if needed
+                        if let Some(MaybeIdReference::Id(id)) =
+                            &request.arguments.on_success_activate_script
+                            && document_id == id.document_id()
+                        {
+                            result.insert_unchecked(SieveProperty::IsActive, true);
+                        }
+
+                        // Add result
                         ctx.response.updated.append(
                             id,
-                            blob_id.map(|blob_id| {
-                                Value::Object(Map::with_capacity(1).with_key_value(
-                                    SieveProperty::BlobId,
-                                    SieveValue::BlobId(blob_id),
-                                ))
-                            }),
+                            if !result.is_empty() {
+                                Value::Object(result).into()
+                            } else {
+                                None
+                            },
                         );
                     }
                     Err(err) => {
@@ -265,87 +295,68 @@ impl SieveScriptSet for Server {
         }
 
         // Process deletions
+        let active_script_id = self.sieve_script_get_active_id(account_id).await?;
         for id in will_destroy {
             let document_id = id.document_id();
             if sieve_ids.contains(document_id) {
-                match self
-                    .sieve_script_delete(&ctx.resource_token, document_id, true, &mut batch)
-                    .await?
-                {
-                    Some(true) => {
+                if active_script_id != Some(document_id) {
+                    if self
+                        .sieve_script_delete(account_id, document_id, ctx.access_token, &mut batch)
+                        .await?
+                    {
                         ctx.response.destroyed.push(id);
-                    }
-                    Some(false) => {
-                        ctx.response.not_destroyed.append(
-                            id,
-                            SetError::new(SetErrorType::ScriptIsActive)
-                                .with_description("Deactivate Sieve script before deletion."),
-                        );
-                    }
-                    None => {
+                    } else {
                         ctx.response.not_destroyed.append(id, SetError::not_found());
                     }
+                } else {
+                    ctx.response.not_destroyed.append(
+                        id,
+                        SetError::new(SetErrorType::ScriptIsActive)
+                            .with_description("Deactivate Sieve script before deletion."),
+                    );
                 }
             } else {
                 ctx.response.not_destroyed.append(id, SetError::not_found());
             }
         }
 
-        // Write changes
-        if !batch.is_empty() {
-            let change_id = self
-                .commit_batch(batch)
-                .await
-                .and_then(|ids| ids.last_change_id(account_id))
-                .caused_by(trc::location!())?;
-            ctx.response.new_state = State::Exact(change_id).into();
-        }
-
         // Activate / deactivate scripts
+        let on_success_deactivate_script = request
+            .arguments
+            .on_success_deactivate_script
+            .unwrap_or(false);
         if ctx.response.not_created.is_empty()
             && ctx.response.not_updated.is_empty()
             && ctx.response.not_destroyed.is_empty()
             && (request.arguments.on_success_activate_script.is_some()
-                || request
-                    .arguments
-                    .on_success_deactivate_script
-                    .unwrap_or(false))
+                || on_success_deactivate_script)
         {
-            let (change_id, changed_ids) =
-                if let Some(id) = request.arguments.on_success_activate_script {
-                    self.sieve_activate_script(
-                        account_id,
-                        match id {
-                            MaybeIdReference::Id(id) => id.document_id(),
-                            MaybeIdReference::Reference(id_ref) => {
-                                match ctx.response.get_created_id(&id_ref) {
-                                    Some(AnyId::Id(id)) => id.document_id(),
-                                    _ => return Ok(ctx.response),
-                                }
-                            }
-                            MaybeIdReference::Invalid(_) => return Ok(ctx.response),
-                        }
-                        .into(),
-                    )
-                    .await?
-                } else {
-                    self.sieve_activate_script(account_id, None).await?
-                };
-
-            if !changed_ids.is_empty() {
-                for (document_id, is_active) in changed_ids {
-                    if let Some(Value::Object(obj)) =
-                        ctx.response.get_object_by_id(Id::from(document_id))
-                    {
-                        obj.insert(SieveProperty::IsActive, Value::Bool(is_active));
-                    }
-                }
-                if change_id > 0 {
-                    ctx.response.new_state = State::Exact(change_id).into();
-                }
+            if let Some(MaybeIdReference::Id(id)) = request.arguments.on_success_activate_script {
+                batch
+                    .with_account_id(account_id)
+                    .with_collection(Collection::Principal)
+                    .with_document(0)
+                    .set(PrincipalField::ActiveScriptId, id.document_id().serialize());
+            } else if on_success_deactivate_script {
+                batch
+                    .with_account_id(account_id)
+                    .with_collection(Collection::Principal)
+                    .with_document(0)
+                    .clear(PrincipalField::ActiveScriptId);
             }
         }
 
+        // Write changes
+        if !batch.is_empty()
+            && let Ok(change_id) = self
+                .commit_batch(batch)
+                .await
+                .caused_by(trc::location!())?
+                .last_change_id(account_id)
+        {
+            ctx.response.new_state = State::Exact(change_id).into();
+        }
+
         Ok(ctx.response)
     }
 
@@ -402,13 +413,13 @@ impl SieveScriptSet for Server {
                         .as_ref()
                         .is_none_or(|(_, obj)| obj.inner.name != value.as_ref())
                         && let Some(id) = self
-                            .filter(
+                            .document_ids_matching(
                                 ctx.resource_token.account_id,
                                 Collection::SieveScript,
-                                vec![Filter::eq(SieveField::Name, value.as_bytes().to_vec())],
+                                SieveField::Name,
+                                value.as_bytes(),
                             )
                             .await?
-                            .results
                             .min()
                     {
                         return Ok(Err(SetError::already_exists()
@@ -448,9 +459,6 @@ impl SieveScriptSet for Server {
                     .map(char::from)
                     .collect::<String>();
             }
-
-            // Set script as inactive
-            changes.is_active = false;
         }
 
         let blob_update = if let Some(blob_id) = blob_id {
