@@ -683,10 +683,12 @@ impl From<Rows> for Vec<u32> {
 }
 
 impl Store {
+    #[inline(always)]
     pub fn is_none(&self) -> bool {
         matches!(self, Self::None)
     }
 
+    #[inline(always)]
     pub fn is_sql(&self) -> bool {
         match self {
             #[cfg(feature = "sqlite")]
@@ -705,6 +707,7 @@ impl Store {
         }
     }
 
+    #[inline(always)]
     pub fn is_pg_or_mysql(&self) -> bool {
         match self {
             #[cfg(feature = "mysql")]
@@ -715,6 +718,15 @@ impl Store {
         }
     }
 
+    #[inline(always)]
+    pub fn is_foundationdb(&self) -> bool {
+        match self {
+            #[cfg(feature = "foundation")]
+            Store::FoundationDb(_) => true,
+            _ => false,
+        }
+    }
+
     // SPDX-SnippetBegin
     // SPDX-FileCopyrightText: 2020 Stalwart Labs LLC <hello@stalw.art>
     // SPDX-License-Identifier: LicenseRef-SEL
