@@ -4,7 +4,7 @@
  * SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-SEL
  */
 
-use crate::jmap::JMAPTest;
+use crate::jmap::{JMAPTest, wait_for_index};
 use jmap_client::mailbox::Role;
 
 pub async fn test(params: &mut JMAPTest) {
@@ -35,6 +35,8 @@ pub async fn test(params: &mut JMAPTest) {
         expected_result[num - 1] = email.take_id();
     }
 
+    wait_for_index(&params.server).await;
+
     assert_eq!(
         client
             .thread_get(&thread_id)
