@@ -16,6 +16,7 @@ use crate::{
         JMAPTest, ManagementApi,
         mail::delivery::{AssertResult, SmtpConnection},
         server::List,
+        wait_for_index,
     },
 };
 use common::{
@@ -24,19 +25,29 @@ use common::{
     core::BuildServer,
     enterprise::{
         Enterprise, MetricStore, TraceStore, Undelete, config::parse_metric_alerts,
-        license::LicenseKey, undelete::DeletedBlob,
+        license::LicenseKey,
     },
     telemetry::{
         metrics::store::{Metric, MetricsStore, SharedMetricHistory},
-        tracers::store::{TracingQuery, TracingStore},
+        tracers::store::TracingStore,
     },
 };
-use http::management::enterprise::undelete::{UndeleteRequest, UndeleteResponse};
+use directory::{QueryBy, backend::internal::manage::ManageDirectory};
+use http::management::{
+    enterprise::undelete::{
+        DeletedBlobResponse, DeletedItemResponse, UndeleteRequest, UndeleteResponse,
+    },
+    stores::destroy_account_data,
+};
 use imap_proto::ResponseType;
+use nlp::language::Language;
 use std::{sync::Arc, time::Duration};
 use store::{
     rand::{self, Rng},
-    write::now,
+    search::{
+        SearchField, SearchFilter, SearchOperator, SearchQuery, SearchValue, TracingSearchField,
+    },
+    write::{SearchIndex, now},
 };
 use trc::{
     ipc::{bitset::Bitset, subscriber::SubscriberBuilder},
@@ -128,7 +139,7 @@ pub async fn test(params: &mut JMAPTest) {
 
     // Create test account
     let server = params.server.inner.build_server();
-    server
+    let account_id = server
         .store()
         .create_test_user(
             "jdoe@example.com",
@@ -143,6 +154,17 @@ pub async fn test(params: &mut JMAPTest) {
     tracing(params).await;
     metrics(params).await;
 
+    // Delete test account
+    server
+        .store()
+        .delete_principal(QueryBy::Id(account_id))
+        .await
+        .unwrap();
+    destroy_account_data(&server, account_id, true)
+        .await
+        .unwrap();
+    params.assert_is_empty().await;
+
     params.server.inner.shared_core.store(
         params
             .server
@@ -234,6 +256,7 @@ async fn alerts(server: &Server) {
 async fn tracing(params: &mut JMAPTest) {
     // Enable tracing
     let store = params.server.core.storage.data.clone();
+    let query = params.server.core.storage.fts.clone();
     TelemetrySubscriberType::StoreTracer(StoreTracer {
         store: store.clone(),
     })
@@ -243,16 +266,19 @@ async fn tracing(params: &mut JMAPTest) {
     );
 
     // Make sure there are no span entries in the db
-    store.purge_spans(Duration::from_secs(0)).await.unwrap();
+    store
+        .purge_spans(Duration::from_secs(0), Some(&query))
+        .await
+        .unwrap();
     assert_eq!(
-        store
-            .query_spans(
-                &[TracingQuery::EventType(EventType::Smtp(
-                    SmtpEvent::ConnectionStart
-                ))],
-                0,
-                0
-            )
+        query
+            .query_global(SearchQuery::new(SearchIndex::Tracing).with_filters(vec![
+                SearchFilter::Operator {
+                    field: SearchField::Tracing(TracingSearchField::EventType),
+                    op: SearchOperator::Equal,
+                    value: SearchValue::Uint(EventType::Smtp(SmtpEvent::ConnectionStart).code())
+                }
+            ]))
             .await
             .unwrap(),
         Vec::<u64>::new()
@@ -275,18 +301,30 @@ async fn tracing(params: &mut JMAPTest) {
     )
     .await;
     lmtp.quit().await;
-    tokio::time::sleep(Duration::from_millis(200)).await;
+    tokio::time::sleep(Duration::from_millis(300)).await;
+
+    params.server.notify_task_queue();
+    wait_for_index(&params.server).await;
 
     // Purge should not delete anything at this point
-    store.purge_spans(Duration::from_secs(1)).await.unwrap();
+    store
+        .purge_spans(Duration::from_secs(2), Some(&query))
+        .await
+        .unwrap();
 
     // There should be a span entry in the db
     for span_type in [
         EventType::Delivery(DeliveryEvent::AttemptStart),
         EventType::Smtp(SmtpEvent::ConnectionStart),
     ] {
-        let spans = store
-            .query_spans(&[TracingQuery::EventType(span_type)], 0, 0)
+        let spans = query
+            .query_global(SearchQuery::new(SearchIndex::Tracing).with_filters(vec![
+                SearchFilter::Operator {
+                    field: SearchField::Tracing(TracingSearchField::EventType),
+                    op: SearchOperator::Equal,
+                    value: SearchValue::Uint(span_type.code()),
+                },
+            ]))
             .await
             .unwrap();
         assert_eq!(spans.len(), 1, "{span_type:?}");
@@ -298,30 +336,44 @@ async fn tracing(params: &mut JMAPTest) {
 
     // Try searching
     for keyword in ["bill@example.com", "jdoe@example.com", "example.com"] {
-        let spans = store
-            .query_spans(&[TracingQuery::Keywords(keyword.to_string())], 0, 0)
+        let spans = query
+            .query_global(SearchQuery::new(SearchIndex::Tracing).with_filters(vec![
+                SearchFilter::Operator {
+                    field: SearchField::Tracing(TracingSearchField::Keywords),
+                    op: SearchOperator::Equal,
+                    value: SearchValue::Text {
+                        value: keyword.to_string(),
+                        language: Language::None,
+                    },
+                },
+            ]))
             .await
             .unwrap();
+
         assert_eq!(spans.len(), 2, "keyword: {keyword}");
-        assert!(spans[0] > spans[1], "keyword: {keyword}");
+        assert!(spans[0] != spans[1], "keyword: {keyword}");
     }
 
     // Purge should delete the span entries
     tokio::time::sleep(Duration::from_millis(800)).await;
-    store.purge_spans(Duration::from_secs(1)).await.unwrap();
+    store
+        .purge_spans(Duration::from_secs(1), Some(&query))
+        .await
+        .unwrap();
 
-    for query in [
-        TracingQuery::EventType(EventType::Smtp(SmtpEvent::ConnectionStart)),
-        TracingQuery::EventType(EventType::Delivery(DeliveryEvent::AttemptStart)),
-        TracingQuery::Keywords("bill@example.com".to_string()),
-        TracingQuery::Keywords("jdoe@example.com".to_string()),
-        TracingQuery::Keywords("example.com".to_string()),
-    ] {
-        assert_eq!(
-            store.query_spans(&[query], 0, 0).await.unwrap(),
-            Vec::<u64>::new()
-        );
-    }
+    assert_eq!(
+        query
+            .query_global(SearchQuery::new(SearchIndex::Tracing).with_filters(vec![
+                SearchFilter::Operator {
+                    field: SearchField::Id,
+                    op: SearchOperator::GreaterThan,
+                    value: SearchValue::Uint(0),
+                },
+            ]))
+            .await
+            .unwrap(),
+        Vec::<u64>::new()
+    );
 }
 
 async fn metrics(params: &mut JMAPTest) {
@@ -344,7 +396,7 @@ async fn metrics(params: &mut JMAPTest) {
     );
 }
 
-async fn undelete(_params: &mut JMAPTest) {
+async fn undelete(params: &mut JMAPTest) {
     // Authenticate
     let mut imap = ImapConnection::connect(b"_x ").await;
     imap.authenticate("jdoe@example.com", "12345").await;
@@ -397,15 +449,25 @@ async fn undelete(_params: &mut JMAPTest) {
     api.get::<serde_json::Value>("/api/store/purge/account/jdoe@example.com")
         .await
         .unwrap();
+    wait_for_index(&params.server).await;
     tokio::time::sleep(Duration::from_millis(200)).await;
     let deleted = api
-        .get::<List<DeletedBlob<String, String, String>>>("/api/store/undelete/jdoe@example.com")
+        .get::<List<DeletedBlobResponse>>("/api/store/undelete/jdoe@example.com")
         .await
         .unwrap()
         .unwrap_data()
         .items;
     assert_eq!(deleted.len(), 1);
     let deleted = deleted.into_iter().next().unwrap();
+    match deleted.item {
+        DeletedItemResponse::Email { from, subject, .. } => {
+            assert_eq!(subject.as_ref(), "undelete test");
+            assert_eq!(from.as_ref(), "john@example.com");
+        }
+        other => {
+            panic!("Unexpected deleted item response: {:?}", other);
+        }
+    }
 
     // Undelete
     let result = api
@@ -413,7 +475,7 @@ async fn undelete(_params: &mut JMAPTest) {
             "/api/store/undelete/jdoe@example.com",
             &vec![UndeleteRequest {
                 hash: deleted.hash,
-                collection: deleted.collection,
+                collection: "email".to_string(),
                 time: deleted.deleted_at,
                 cancel_deletion: deleted.expires_at.into(),
             }],
