@@ -175,7 +175,7 @@ pub fn new_message(queue_id: u64) -> MessageWrapper {
             flags: 0,
             env_id: None,
             priority: 0,
-            quota_keys: vec![],
+            quota_keys: Default::default(),
             blob_hash: Default::default(),
             received_from_ip: IpAddr::V4(Ipv4Addr::LOCALHOST),
             received_via_port: 0,
@@ -200,10 +200,11 @@ fn next_event_after(message: &Message, queue: Option<QueueName>, instant: u64) -
             {
                 next_event = rcpt.notify.due.into();
             }
-            if let Some(expires) = rcpt.expiration_time(message.created) {
-                if expires > instant && next_event.as_ref().is_none_or(|ne| expires.lt(ne)) {
-                    next_event = expires.into();
-                }
+            if let Some(expires) = rcpt.expiration_time(message.created)
+                && expires > instant
+                && next_event.as_ref().is_none_or(|ne| expires.lt(ne))
+            {
+                next_event = expires.into();
             }
         }
     }
